close all
clear all
% Physical parameters
m = 0.4
l = 0.15
b = 0.005
g = 10
k = 0.01

% Model parameters (Theta0)
a1 = b/m/l/l
a0 = g/l
b0 = k/m/l/l

K = 1
T = 0.01 % Sampling time
phi0 = pi/4 % starting angle

Tsim = 10 %Simulation time

s = tf('s')
C = 179.36 *(s+2.093) *(s+45.23)/s /(s+171.5) %Controller in simulink


% Excitation signal
w = linspace(0.1,5,10)
A = linspace(1,0.1,10)*0.3


%% Central difference
sim('ClosedLoopIdentSol.slx')
% arx model

N = min([length(y_sim.Data) length(u_sim.Data)])

clear y u 
% Simulink data load
y = y_sim.Data(1:N)
u = u_sim.Data(1:N)


% Calculates the first and second derivatives using central difference
start = 2
for k = start:N-1
   ydot(k) = (y(k+1)-y(k-1))/(2*T); 
   yddot(k) = (y(k+1)-2*y(k)+y(k-1))/(T^2);
end

% Measured derivatives
ydotsim = ydot_sim.Data(1:N);
yddotsim = ydotdot_sim.Data(1:N);

% figure
% hold on
% plot(1:length(ydotsim),ydotsim)
% plot(1:length(ydot),ydot)
% 
% figure
% hold on
% plot(1:length(yddotsim),yddotsim)
% plot(1:length(yddot),yddot)


%% Filter parameters 
order_fir = 20        % FIR filter order
Wn_fir = 1*2*pi % Cutoff frequency (rad/s)

order_iir = 2
Wn_iir = 2*pi % Cutoff frequency

%% FIR filter

Fp  = Wn_fir/(2*pi)
%Fp = 1
Fs  = 1/T;       
h = fir1(order_fir,Fp/(Fs/2))  % Calculated FIR filter

% Filters the derivative data
for k = start+(order_fir/2):N-(order_fir/2)-2
    yfd = 0;
    ydotfd = 0;
    yddotfd = 0;
    for i = 1:order_fir+1
        yfd = yfd + h(i) * y(k - (order_fir/2) + i);
        ydotfd = ydotfd + h(i) * ydot(k - (order_fir/2) + i);
        yddotfd = yddotfd + h(i) * yddot(k - (order_fir/2) + i);
    end
    yf_fir(k) = yfd;
    ydotf_fir(k) = ydotfd;
    yddotf_fir(k) = yddotfd;
%     yf(k) = B(1) * y(k-1) + B(2) * y(k) + B(3) * y(k+1);
%     ydotf(k) = B(1) * ydot(k-1) + B(2) * ydot(k) + B(3) * ydot(k+1); 
%     yddotf(k) = B(1) * yddot(k-1) + B(2) * yddot(k) + B(3) * yddot(k+1); 
end


% figure 
% hold on
% plot(y)
% plot(yf_fir)
% legend('y', 'yf')
% 
% figure 
% hold on
% plot(ydot)
% plot(ydotsim)
% plot(ydotf_fir)
% legend('ydot', 'ydotsim', 'ydotf')
% 
% figure 
% hold on
% plot(yddot)
% plot(yddotsim)
% plot(yddotf_fir)
% legend('yddot', 'yddotsim', 'yddotf')


%% IIR filter
close all

fs = 1/T
fp = Wn_iir/(2*pi)

d = designfilt('lowpassiir','FilterOrder',order_iir, ...
    'PassbandFrequency',fp,'PassbandRipple',0.2, ...
         'SampleRate',1/T);
%fvtool(d)


%[B, A] = butter(order_iir,fp/(fs/2))
%G = tf(B,A,T)
% [z,p,k] = butter(order_iir,fp/(fs/2))
% G = zpk(z,p,k,T)
% fvtool(ss(G),d,'Fs',1/T)
% Gz = c2d(G,T)
% B = Gz.Numerator{1}
% A = Gz.Denominator{1}

% Filters the derivative data
% yf_iir = filtfilt(B,A,y);
% ydotf_iir = filtfilt(B,A,ydot);
% yddotf_iir = filtfilt(B,A,yddot);

% yf_iir = filtfilt(G,y);
% ydotf_iir = filtfilt(G,ydot);
% yddotf_iir = filtfilt(G,yddot);

yf_iir = filtfilt(d,y);
ydotf_iir = filtfilt(d,ydot);
yddotf_iir = filtfilt(d,yddot);

%% Porovnani

e_iir = mean((y - yf_iir).^2)
edot_iir = mean((ydotsim(1:end-1)' - ydotf_iir).^2)
eddot_iir = mean((yddotsim(1:end-1)' - yddotf_iir).^2)

e_fir = mean((y(1:min(length(y),length(yf_fir)))' - yf_fir(1:min(length(y),length(yf_fir)))).^2)
edot_fir = mean((ydot(1:min(length(y),length(yf_fir))) - ydotf_fir(1:min(length(y),length(yf_fir)))).^2)
eddot_fir = mean((yddot(1:min(length(y),length(yf_fir))) - yddotf_fir(1:min(length(y),length(yf_fir)))).^2)

pkpky = max(y)-min(y)
pkpkydot = max(ydotsim)-min(ydotsim)
pkpkyddot = max(yddotsim)-min(yddotsim)

pkpky_iir = max(yf_iir)-min(yf_iir)
pkpkydot_iir = max(ydotf_iir)-min(ydotf_iir)
pkpkyddot_iir = max(yddotf_iir)-min(yddotf_iir)

pkpky_fir = max(yf_fir)-min(yf_fir)
pkpkydot_fir = max(ydotf_fir)-min(ydotf_fir)
pkpkyddot_fir = max(yddotf_fir)-min(yddotf_fir)

fprintf('         e_y          e_ydot           e_yddot         pk-pk_y        pk-pk_ydot      pk-pk_yddot \n');
% fprintf('IIR     -    -   pkpky    pkpkydot   pkpkyddot \n');
fprintf('sim   %5d           %5d            %5d         %10d    %10d    %10d\n', NaN, NaN, NaN, pkpky, pkpkydot, pkpkyddot);
fprintf('FIR   %5d    %5d   %5d    %10d    %10d    %10d\n', e_fir, edot_fir, eddot_fir, pkpky_fir, pkpkydot_fir, pkpkyddot_fir);
fprintf('IIR   %5d    %5d   %5d    %10d    %10d    %10d\n', e_iir, edot_iir, eddot_iir, pkpky_iir, pkpkydot_iir, pkpkyddot_iir);
%% 
figure
subplot(1,3,1)
title('IIR')
hold on
plot(y)
plot(yf_iir)
legend('y', 'yf')

subplot(1,3,2)
hold on
plot(ydot)
plot(ydotsim)
plot(ydotf_iir)
legend('ydot', 'ydotsim', 'ydotf')

subplot(1,3,3)
hold on
plot(yddot)
plot(yddotsim)
plot(yddotf_iir)
legend('yddot', 'yddotsim', 'yddotf')

%% 
figure
subplot(1,3,1)
title('FIR')
hold on
plot(y)
plot(yf_fir)
legend('y', 'yf')

subplot(1,3,2)
hold on
plot(ydotsim)
plot(ydotf_fir)
legend('ydotsim', 'ydotf')

subplot(1,3,3)
hold on
plot(yddotsim)
plot(yddotf_fir)
legend('yddotsim', 'yddotf')

%% 

% System identifaction using IIR filtered data
Nmax = min([length(ydotf_iir) length(yf_iir) length(u)]);
start = 3;
for n = start:Nmax
    Phi_iir(n-start+1, :) = [ -ydotf_iir(n)  -sin(yf_iir(n))   u(n)];
end

% Theta_iir = Phi_iir\yddotf_iir(start:Nmax)'
Theta_iir = inv(Phi_iir'*Phi_iir)*Phi_iir'*yddotf_iir(start:Nmax)'

H = Phi_iir*inv(Phi_iir'*Phi_iir)*Phi_iir';

%epsilon_iir = (eye(length(H))-H)*y(1:Nmax);
epsilon_iir = (eye(length(H))-H)*yddotf_iir(start:Nmax)';
mean_iir = mean(epsilon_iir)

s2_iir = ((epsilon_iir-mean_iir)'*(epsilon_iir-mean_iir))/(Nmax-start-length(Theta_iir)) % Estimate of covariance of epsilon

cove_iir = s2_iir*(eye(length(H))-H); % Covariance matrix of epsilon

cov_theta_iir = s2_iir * inv(Phi_iir'*Phi_iir); % Covariance matrix of Theta

% sqrt(cov_theta_iir(1,1))/Theta_iir(1)
% sqrt(cov_theta_iir(2,2))/Theta_iir(2)
% sqrt(cov_theta_iir(3,3))/Theta_iir(3)

% System identifaction using FIR filtered data
Nmax = min([length(ydotf_fir) length(yf_fir) length(u)]);
start = 3;
for n = start:Nmax
    Phi_fir(n-start+1, :) = [ -ydotf_fir(n)  -sin(yf_fir(n))   u(n)];
end


% Theta_fir = Phi_fir\yddotf_fir(start:Nmax)'
Theta_fir = inv(Phi_fir'*Phi_fir)*Phi_fir'*yddotf_fir(start:Nmax)'
% epsilon_fir = (yddotf_fir(start:Nmax)' - Phi_fir*Theta_fir);

yhat = Phi_iir*Theta_iir;
H = Phi_fir*inv(Phi_fir'*Phi_fir)*Phi_fir';

epsilon_fir = (eye(length(H))-H)*yddotf_fir(start:Nmax)';
mean_fir = mean(epsilon_fir)


s2_fir = ((epsilon_fir-mean_fir)'*(epsilon_fir-mean_fir))/(Nmax-length(Theta_fir))

cove_fir = s2_fir*(eye(length(H))-H);

cov_theta_fir = s2_fir * inv(Phi_fir'*Phi_fir)

sqrt(cov_theta_fir(1,1))/Theta_fir(1)
sqrt(cov_theta_fir(2,2))/Theta_fir(2)
sqrt(cov_theta_fir(3,3))/Theta_fir(3)

Theta_0 = [a1 a0 b0]' % True model parameters

%% fft


% P2 = abs(Y/length(y));
% P1 = P2(1: floor((length(y)/2+1)));
% P1(2:end-1) = 2*P1(2:end-1);
L = length(y);
NFFT = 2^nextpow2(L);

f_res = 1/T/length(y);
f = (0:length(y)-1)*f_res;

f = 1/T/2*linspace(0,1,NFFT/2+1);
Y = fft(y,NFFT)/L;
Ymag = abs(Y);
figure
plot(f,Ymag(1:NFFT/2+1))
title('Single-Sided Amplitude Spectrum of y(t)')
xlabel('f (Hz)')
ylabel('|P1(f)|')
grid on

% <0,1>Hz most energy
% <0, 2pi> rad/s

%%
% clear Phi
% Nmax = min([length(ydotsim) length(y) length(u)]);
% start = 5;
% for n = start:Nmax
%     Phi(n-start+1, :) = [ -ydotsim(n)  -sin(y(n))   u(n)];
% end
% Theta = Phi\yddotsim(start:Nmax)
% 
% epsilon = yddotsim(start:Nmax) - Phi*Theta_0;
% mean(epsilon)

%% 
% clear Phi
% Nmax = min([length(ydot) length(y) length(u)]);
% start = 5;
% for n = start:Nmax
%     Phi(n-start+1, :) = [ -ydot(n)  -sin(y(n))   u(n)];
% end
% Theta = Phi\yddot(start:Nmax)'
% 
% epsilon = yddot(start:Nmax)' - Phi*Theta_0;
% mean(epsilon)
